from flask import Flask, request,  jsonify
from flask_cors import CORS

import tensorflow as tf
import numpy as np
import pandas as pd

from sklearn.model_selection import train_test_split
from keras.preprocessing import sequence

from model.UrlClassifier import UrlClassifier
from model.TextCleaner import TextCleaner


MAX_LEN = 255
BATCH_SIZE = 64


def remove_missing_data(data):
    data.isnull().sum()

    return data


def open_csv():
    data = pd.read_csv('./data/fraud-email-dataset/fraud_email_.csv', dtype={'Text': 'string'})
    data.drop_duplicates(inplace=True)
    data = remove_missing_data(data)

    return data


def process_text(text):
    text_cleaner = TextCleaner(text)
    text_cleaned = text_cleaner.get_text_cleaned()

    return text_cleaned.split()


def one_hot_encoding(words):
    global word_encoding

    encoding = []

    for word in words:
        if word in vocab:
            code = vocab[word]
            encoding.append(code)
        else:
            vocab[word] = word_encoding
            encoding.append(word_encoding)
            word_encoding = word_encoding + 1

    return encoding


def get_tokens():
    matrix_of_tokens = data['Text'].apply(lambda x: one_hot_encoding(process_text(x)))

    return matrix_of_tokens


def encode_text(text):
    tokens = one_hot_encoding(process_text(text))
    for index in range(len(tokens)):
        if tokens[index] >= 130494:
            tokens[index] = 0

    return sequence.pad_sequences([tokens], MAX_LEN)[0]


def predict(text):
    encoded_text = encode_text(text)
    pred = np.zeros((1, 255))
    pred[0] = encoded_text
    result = model.predict(pred)

    print(result[0][0])

    return result[0][0]


app = Flask(__name__)
CORS(app)


@app.route("/")
def hello():
    return "Welcome to machine learning model APIs!"


@app.route('/predict_email', methods=['POST'])
def predict_email():
    request_data = request.get_json()

    print(request_data)

    return jsonify({'prediction': str(predict(request_data))})


@app.route('/predict_url', methods=['POST'])
def makecalc():
    request_data = request.get_json()
    request_data_list = list(request_data.values())

    x_predict = vectorizer.transform(request_data_list)
    y_predict = logistic_regression.predict(x_predict)

    print(y_predict[0])

    return jsonify({'prediction': str(y_predict[0])})


if __name__ == '__main__':
    url_classifier = UrlClassifier()
    vectorizer, logistic_regression = url_classifier.tensor_learning()

    vocab = {}
    word_encoding = 1

    data = open_csv()
    tokens = get_tokens()

    train_data, test_data, train_labels, test_labels = train_test_split(tokens, data['Class'], test_size=0.2,
                                                                        random_state=0)
    train_data = sequence.pad_sequences(train_data, MAX_LEN)
    test_data = sequence.pad_sequences(test_data, MAX_LEN)

    total_words = len(vocab) + 1

    model = tf.keras.Sequential([
        tf.keras.layers.Embedding(total_words, 32),
        tf.keras.layers.LSTM(32),
        tf.keras.layers.Dense(1, activation="sigmoid")
    ])

    model.compile(loss="binary_crossentropy", optimizer="rmsprop", metrics=['acc'])

    history = model.fit(train_data, train_labels, epochs=10, validation_split=0.2)
    results = model.evaluate(test_data, test_labels)

    app.run(debug=True)
