"""
Class to Read Information from files.

@Author: Vitor
"""

import os
import pandas as pd

from model.utils import get_urls_in_string


class DataReader:
    def __init__(self, filename):
        self.PATTERN = "From r"
        self.filename = filename
        self.delimiter = None
        self.file_dir = "../data/"
        self.cwd = os.getcwd()
        self.encoding = "ISO-8859-1"
        self.file = None
        self.fraudulent_emails = []
        self.fraudulent_urls = []

    def get_file(self):
        self.file = self.file_dir + self.filename

    def csv_to_dataframe(self,  utf_encode=True, error_bad_lines=True):
        if utf_encode and not error_bad_lines:
            data = pd.read_csv(os.path.join(self.cwd, self.file), error_bad_lines=False)
        elif utf_encode and error_bad_lines:
            data = pd.read_csv(os.path.join(self.cwd, self.file))
        else:
            data = pd.read_csv(os.path.join(self.cwd, self.file), error_bad_lines=False, encoding="ISO-8859-1")

        return data

    def open_fraud_email_csv(self):
        self.get_file()
        return self.csv_to_dataframe()

    def open_spam_ham_email_csv(self):
        self.get_file()
        return self.csv_to_dataframe(error_bad_lines=False)

    def get_ham_emails(self):
        spam_ham_emails = self.open_spam_ham_email_csv()
        return spam_ham_emails.loc[spam_ham_emails['label'] == 'ham']

    def read_txt_file_data(self):
        self.get_file()
        emails = []

        with open(self.file, encoding="utf8", errors='ignore') as data_file:
            lines = ''
            first_line = True

            for line in data_file:
                self.fraudulent_urls.append(get_urls_in_string(line))

                if line.startswith(self.PATTERN) and not first_line:
                    emails.append(lines)
                    lines = ''
                else:
                    first_line = False
                lines = lines + line
        return emails

    def get_fraudulent_emails(self):
        self.fraudulent_emails = self.read_txt_file_data()

    def open_email_folder(self, folder):
        data = []
        files = [f for f in os.listdir(folder)]

        for file in files:
            with open(folder + file, "r", encoding="ISO-8859-1") as f:
                data.append(f.read())

        df = pd.DataFrame(data)

        return df
