"""
Tokenizer API
So far we have looked at one-off convenience methods for preparing text with Keras.

Keras provides a more sophisticated API for preparing text that can be fit and reused to
prepare multiple text documents. This may be the preferred approach for large projects.

Keras provides the Tokenizer class for preparing text documents for deep learning.
The Tokenizer must be constructed and then fit on either raw text documents or integer
encoded text documents.


@Author: Vitor
"""
import math

from keras.preprocessing.text import Tokenizer


class MyTokenizer:
    def __init__(self, text):
        self.text = text
        self.word_counts = {}
        self.document_count = 0
        self.word_index = {}
        self.word_docs = {}
        self.tokenizer = Tokenizer()
        self.tokens = []
        self.tokenizer_text_array()

    def tokenizer_text_array(self):
        self.tokenizer.fit_on_texts(self.text)
        self.summarize_what_was_learned()
        self.tokens = self.tokenizer.texts_to_matrix(self.text, mode='count')

    def summarize_what_was_learned(self):
        self.word_counts = self.tokenizer.word_counts
        self.document_count = self.tokenizer.document_count
        self.word_index = self.tokenizer.word_index
        self.word_docs = self.tokenizer.word_docs

    def get_tokens(self):
        return self.tokens
