"""


@Author: Vitor
"""


import numpy as np
import pandas as pd

from model.UrlTokenizer import UrlTokenizer

from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.model_selection import train_test_split
from sklearn.linear_model import LogisticRegression


class UrlClassifier:

    def __init__(self):
        self.RANDOM_STATE = 42
        self.TEST_SIZE = 0.2

        self.url_tokenizer = UrlTokenizer()

    def tensor_learning(self):
        all_urls_data = self.get_all_urls_data()
        self.shuffle_array(all_urls_data)

        urls_label = self.get_urls_label(all_urls_data)
        urls_corpus = self.get_urls_corpus(all_urls_data)

        vectorizer = TfidfVectorizer(tokenizer=self.url_tokenizer.get_tokens)
        x_vector = self.get_x_vector(vectorizer, urls_corpus)

        x_train, x_test, y_train, y_test = train_test_split(x_vector,
                                                            urls_label,
                                                            test_size=self.TEST_SIZE,
                                                            random_state=self.RANDOM_STATE)

        logistic_regression = LogisticRegression(max_iter=1000, dual=False)
        logistic_regression.fit(x_train, y_train)
        print(logistic_regression.score(x_test, y_test))

        return vectorizer, logistic_regression

    def get_all_urls_data(self):
        all_urls_dataframe = self.get_all_urls_dataframe()
        all_urls_data = self.dataframe_to_array(all_urls_dataframe)

        return all_urls_data

    def get_x_vector(self, vectorizer, urls_corpus):
        x_vector = vectorizer.fit_transform(urls_corpus)

        return x_vector

    def get_all_urls_dataframe(self):
        all_urls_path = './data/data.csv'
        all_urls_csv = pd.read_csv(all_urls_path, ',', error_bad_lines=False)
        all_urls_data = pd.DataFrame(all_urls_csv)

        return all_urls_data

    def dataframe_to_array(self, all_urls_dataframe):
        array_data = np.array(all_urls_dataframe)

        return array_data

    def shuffle_array(self, array_to_shuffle):
        np.random.shuffle(array_to_shuffle)

    def get_urls_label(self, urls_with_label):
        return [d[1] for d in urls_with_label]

    def get_urls_corpus(self, urls_with_corpus):
        return [d[0] for d in urls_with_corpus]
