"""


@Author: Vitor
"""


import math
from collections import Counter


class UrlTokenizer:
    def __init__(self):
        self.tokens = []

    def entropy(self, string):
        chars_count_dict = Counter(string)
        string_len = len(string)

        etp = -sum(char_count / string_len * math.log(char_count / string_len, 2)
                   for char_count in chars_count_dict.values())

        return etp

    def get_tokens(self, urls_with_labels):
        tokens_split_by_slash = self.get_string_value_of_split_url_using_slash(urls_with_labels)
        all_tokens = self.get_customized_tokens(tokens_split_by_slash)
        all_tokens = self.remove_redundant_tokens(all_tokens)
        all_tokens = self.remove_sub_string_com_from_tokens(all_tokens)

        return all_tokens

    def get_customized_tokens(self, tokens):
        all_tokens = []

        for token in tokens:
            tokens_split_dash = self.get_tokens_split_by_dash(token)
            tokens_by_dot = self.get_tokens_split_by_dot(tokens_split_dash)
            aux_tokens_hyphen = self.get_tokens_split_by_hyphen(tokens_by_dot)
            all_tokens = all_tokens + tokens_split_dash + aux_tokens_hyphen

        return all_tokens

    def get_string_value_of_split_url_using_slash(self, string):
        return str(string.encode('utf-8')).split('/')

    def remove_redundant_tokens(self, tokens):
        return list(set(tokens))

    def get_tokens_split_by_dash(self, tokens):
        return str(tokens).split('-')

    def get_tokens_split_by_hyphen(self, tokens):
        tokens_by_hyphen = []

        for index in range(0, len(tokens)):
            aux_tokens_hyphen = str(tokens[index]).rsplit('_')
            tokens_by_hyphen = tokens_by_hyphen + aux_tokens_hyphen

        return tokens_by_hyphen

    def get_tokens_split_by_dot(self, tokens):
        tokens_by_dot = []

        for index in range(0, len(tokens)):
            aux_tokens_dot = self.split_by_dot(tokens[index])
            tokens_by_dot = tokens_by_dot + aux_tokens_dot

        return tokens_by_dot

    def split_by_dot(self, tokens):
        return str(tokens).split('.')

    def remove_sub_string_com_from_tokens(self, tokens):
        if self.there_is_substring_com_in_tokens(tokens):
            tokens.remove('com')

        return tokens

    def there_is_substring_com_in_tokens(self, tokens):
        return 'com' in tokens
