"""
Stemming or Lemmatization: stemming is the
process of reducing derived words to their root.
Lemmatization is the process of reducing a word to its lemma.
The main difference between both methods is that
lemmatization provides existing words, whereas stemming provides
the root, which may not be an existing word. We have used a Lemmatizer
based in WordNet.



@Author: Vitor
"""

from nltk.stem import WordNetLemmatizer


class WordCleaner:
    def __init__(self):
        self.lemmatizer = WordNetLemmatizer()

    def clean_word(self, word):
        alphanumeric = ""

        if self.detect_possessive_pronoun(word):
            word = self.remove_possessive_pronoun(word)

        word = self.lemmatize_word(word)

        word_length = len(word)

        for character, index in zip(word, range(0, word_length)):
            if character.isalnum():
                alphanumeric = alphanumeric + self.downcase_char(character)
            if character == " " and index != word_length:
                alphanumeric = alphanumeric + character

        return alphanumeric

    def lemmatize_word(self, word):
        return self.lemmatizer.lemmatize(word)

    def downcase_char(self, char_value):
        return char_value.lower()

    def detect_possessive_pronoun(self, word):
        return word.endswith("’s")

    def remove_possessive_pronoun(self, word):
        return word.replace("’s", "")
